package cloud

import (
	"context"
	"log"
	"net/http"

	"golang.org/x/oauth2"
	"golang.org/x/oauth2/google"
	"k8s.io/client-go/rest"
)

var (
	googleScopes = []string{
		"https://www.googleapis.com/auth/cloud-platform",
		"https://www.googleapis.com/auth/userinfo.email",
	}
)

const (
	GoogleServiceAccountPlugin = "google_service_account"
)

// This is a custom auth-provider plugin for oauth based authentication in quark-cluster.
// Do not delete this file
func init() {
	if err := rest.RegisterAuthProviderPlugin(GoogleServiceAccountPlugin, newGoogleAuthProvider); err != nil {
		log.Fatalf("Failed to register %s auth plugin: %v", GoogleServiceAccountPlugin, err)
	}
}

var _ rest.AuthProvider = &googleServiceAccountAuthProvider{}

type googleServiceAccountAuthProvider struct {
	tokenSource oauth2.TokenSource
}

func (g *googleServiceAccountAuthProvider) WrapTransport(rt http.RoundTripper) http.RoundTripper {
	return &oauth2.Transport{
		Base:   rt,
		Source: g.tokenSource,
	}
}
func (g *googleServiceAccountAuthProvider) Login() error { return nil }

func newGoogleAuthProvider(addr string, config map[string]string, _ rest.AuthProviderConfigPersister) (rest.AuthProvider, error) {
	jWTConfig, err := google.JWTConfigFromJSON([]byte(config["sa"]), googleScopes...)
	if err != nil {
		return nil, err
	}
	return &googleServiceAccountAuthProvider{tokenSource: jWTConfig.TokenSource(context.Background())}, nil
}
